﻿using System.Diagnostics;
using System.Net;
using System.Net.Sockets;
using System.Text;

try
{
    string IPADDRESS = "255.255.255.255";                               // Broadcast address , change it to your own
    if (args.Length > 0) IPADDRESS = args[0];                           // if a start paramter is given , use that as broadcast IP
    Console.WriteLine("Discover CT-TCP/IP interfaces (Broadcast IP "+ IPADDRESS+")...");
    int PORT = 55555;                                                   // UDP port for broadcasting
    UdpClient udpClient = new UdpClient();                              // create UDP client
    udpClient.Client.Bind(new IPEndPoint(IPAddress.Any, PORT));         // bind UDP client to broadcast port
    udpClient.Client.ReceiveTimeout = 50;                               // receive timeout 50ms
    var from = new IPEndPoint(0, 0);                                    // create network endpoint
    var data = Encoding.UTF8.GetBytes("IR-MODUL-BROADCAST");            // broadcast message
    udpClient.Send(data, data.Length, IPADDRESS, PORT);                 // send broadcast command to broadcast IP / port 
    byte[] rb ;                                                         // receive buffer
    UInt16 count = 0;                                                   // device count
    Stopwatch sw = new();                                               // stopwatch for max. search time
    sw.Start();                                                         // start stopwatch
    while (sw.ElapsedMilliseconds < 1000)                               // wait 1000ms for possible responses
    {
        try
        {
            rb = udpClient.Receive(ref from);                   // try to receive a UDP response
            if (rb.Length == 39) {                              // response must tbe 39 bytes long
                if (Encoding.UTF8.GetString(rb).Contains("IR-MODUL-BROADCAST")) // answer must start with "IR-MODUL-BROADCAST"
                {
                    string macaddress   = string.Format("{0:X02}:{1:X02}:{2:X02}:{3:X02}:{4:X02}:{5:X02}", rb[18], rb[19], rb[20], rb[21], rb[22], rb[23]);
                    string staticip     = string.Format("{0}.{1}.{2}.{3}", rb[24], rb[25], rb[26], rb[27]);
                    string ipmode       = rb[28] == 1 ? "DHCP" : "STATIC";
                    string netmask      = string.Format("{0}.{1}.{2}.{3}", rb[29], rb[30], rb[31], rb[32]);
                    int port_cfg        = rb[33] * 256 + rb[34];
                    int port_cmd        = rb[35] * 256 + rb[36];
                    int port_modbus     = rb[37] * 256 + rb[38];
                    Console.WriteLine((++count).ToString("Device : 0"));
                    Console.WriteLine("\t\tcurrent IP :\t" + from.Address.ToString());
                    Console.WriteLine("\t\tMAC address :\t" + macaddress);
                    Console.WriteLine("\t\tstatic IP :\t" + staticip);
                    Console.WriteLine("\t\tIP mode :\t" + ipmode);
                    Console.WriteLine("\t\tNetmask :\t" + netmask);
                    Console.WriteLine("\t\t" + port_cfg.ToString("PORT CFG :\t0"));
                    Console.WriteLine("\t\t" + port_cmd.ToString("PORT CMD :\t0"));
                    Console.WriteLine("\t\t" + port_modbus.ToString("PORT MODBUS :\t0"));
                }
            }
        }
        catch (Exception) {}
    }
    sw.Stop();
    udpClient.Close();
}
catch (Exception e)
{
    Console.WriteLine("Unexpected exception : {0}", e.ToString());
}


